@@ -2,7 +2,7 @@
 
 """
 
-usage: unshorten.py /path/to/your/twitter/archive/directory
+usage: twitter-archive-unshorten.py /path/to/your/twitter/archive/directory
 
 Run this program on an unpacked Twitter Archive directory and it will
 rewrite the t.co short URLs to their unshortened equivalent.
@@ -16,6 +16,7 @@ import os
 import sys
 import json
 import time
+import logging
 import urllib.error
 import urllib.request
 
@@ -29,6 +30,9 @@ def main():
     archive_dir = sys.argv[1]
     sanity_check(archive_dir)
 
+    logging.basicConfig(filename=join(archive_dir, "twitter-archive-unshorten.log"), level=logging.INFO)
+    logging.info("rewriting t.co urls with https://github.com/docnow/twitter-archive-unshorten")
+
     # find all the short urls in the archive
     short_urls = get_short_urls(archive_dir)
 
@@ -94,8 +98,9 @@ def rewrite_files(archive_dir, url_map):
                 # remember the mapping only contains https keys
                 lookup_url = re.sub(r'^http://', 'https://', short_url)
                 if lookup_url in url_map:
-                    rewrote += 1 
+                    logging.info("rewriting {short_url} to {long_url} in {path}")
                     line = line.replace(short_url, url_map[lookup_url])
+                    rewrote += 1 
                 else:
                     print(f"{lookup_url} not found")
             lines.append(line)
@@ -113,25 +118,37 @@ def unshorten(urls, archive_dir):
     urls = set(urls)
 
     # where to write the mapping
-    url_map_file = join(archive_dir, "data", "urlmap.json")
+    url_map_file = join(archive_dir, "data", "shorturls.json")
 
-    # load any mapping data we have already
+    # load any existing mapping data (from a previous run perhaps)
     if os.path.isfile(url_map_file):
         url_map = json.load(open(url_map_file))
     else:
         url_map = {}
 
+    # load short/long mapping already present in tweet.js
+    url_map.update(read_url_map(join(archive_dir, "data", "tweet.js")))
+
     count = 0
     for short_url in urls:
         count += 1
 
         # force https: some old t.co URLs use http
         short_url = re.sub(r'^http://', 'https://', short_url)
+
+        # if we already know what the long url is we can skip it
+        if short_url in url_map:
+            logging.info(f"already have long url for {short_url}")
+            continue
+
+        logging.info(f"looking up {short_url}")
+
         try:
             urllib.request.urlopen(short_url)
         except urllib.error.HTTPError as e:
             if e.code == 301:
                 long_url = e.headers.get('Location')
+                logging.info(f"got {long_url} for {short_url}")
                 # an unescaped " will break JSON serialization
                 long_url = long_url.replace('"', '%22')
                 url_map[short_url] = long_url
@@ -141,7 +158,8 @@ def unshorten(urls, archive_dir):
         print('\r' + msg, end='', flush=True)
 
         # periodically dump the mappings we have
-        if len(url_map) % 10 == 0:
+        if archive_dir != "" and len(url_map) % 10 == 0:
+            logging.info(f"writing {len(url_map)} urls to {url_map_file}")
             json.dump(url_map, open(url_map_file, "w"), indent=2)
 
         # try not to awaken the dragon
@@ -150,9 +168,28 @@ def unshorten(urls, archive_dir):
     print()
     return url_map
 
+def read_url_map(path):
+    """Read short/long mapping in existing data.
+    """
+    text = open(path).read()
+    text = re.sub(r'^window.YTD.tweet.part0 = ', '', text)
+    data = json.loads(text)
+    url_map = {}
+    for tweet in data:
+        entities = tweet['tweet']['entities']['urls']
+        entities.extend(tweet['tweet']['entities'].get('media', []))
+        for url in tweet['tweet']['entities']['urls']:
+            short_url = re.sub(r'^http://', 'https://', url['url'])
+            if short_url.startswith('https://t.co/'):
+                url_map[short_url] = url['expanded_url']
+            
+    return url_map
+
 # Some shenanigans so urllib gets the redirect but doesn't follow it.
 # It would be nice to be able to use requests here but I didn't want to
 # make people install anything extra.
+#
+# It might be worth revisiting this since it is pip installed now.
 
 class NoRedirect(urllib.request.HTTPRedirectHandler):
     def redirect_request(self, *_):
